// Copyright (C) 2010 - Michael Baudin
//
// This file must be used under the terms of the CeCILL.
// This source file is licensed as described in the file COPYING, which
// you should have received as part of this distribution.  The terms
// are also available at
// http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt

function errmsg = specfun_checkflint ( funname , var , varname , ivar )
  // Checks that the argument is a floating point integer.
  //
  // Calling Sequence
  //   errmsg = specfun_checkflint ( var )
  //
  // Parameters
  //   funname : a 1 x 1 matrix of strings, the name of the calling function.
  //   var : a 1 x 1 matrix of doubles, the variable
  //   varname : a 1 x 1 matrix of string, the name of the variable
  //   ivar : a 1 x 1 matrix of floating point integers, the index of the input argument in the calling sequence
  //   errmsg : a 1 x 1 matrix of strings, the error message. If there was no error, the error message is the empty matrix.
  //
  // Description
  // This function is designed to be used to design functions where an 
  // input argument is expected to be a floating point integer, that is, it has 
  // the type "constant", but its content is an integer.
  // The error is generated if the condition or(floor(var)<>var) is true.
  //
  // Examples
  // // The function takes an argument x such that x>=1.
  // function y = myfunction ( x )
  //   specfun_checkflint ( "myfunction" , x , "x" , 1 )
  //   y = 2*x
  // endfunction
  // // Calling sequences which work
  // myfunction ( 2 )
  // // Calling sequences which generate an error
  // myfunction ( 1.1 )
  //
  // Authors
  // Michael Baudin - 2010 
  //

  [lhsnb,rhsnb]=argn()
  if ( rhsnb <> 4 ) then
    msgfmt = "%s: Unexpected number of input arguments : %d provided while %d are expected."
    errmsg = msprintf(gettext(msgfmt), "specfun_checkflint", rhsnb,4)
    error(errmsg)
  end
  //
  // Checking type of input arguments
  if ( typeof(funname) <> "string" ) then
    errmsg = msprintf(gettext("%s: Expected a %s for input argument %s at input #%d, but got %s instead."), "specfun_checkflint", "string" , "funname" , 1 , funname , typeof(funname) );
    error(errmsg)
  end
  if ( typeof(var) <> "constant" ) then
    errmsg = msprintf(gettext("%s: Expected a %s for input argument %s at input #%d, but got [%s] instead."), "specfun_checkflint", "constant" , "var" , 2 , var , typeof(var) );
    error(errmsg)
  end
  if ( typeof(varname) <> "string" ) then
    errmsg = msprintf(gettext("%s: Expected a %s for input argument %s at input #%d, but got [%s] instead."), "specfun_checkflint", "string" , "varname" , 3 , varname , typeof(varname) );
    error(errmsg)
  end
  if ( typeof(ivar) <> "constant" ) then
    errmsg = msprintf(gettext("%s: Expected a %s for input argument %s at input #%d, but got [%s] instead."), "specfun_checkflint", "constant" , "ivar" , 4 , ivar , typeof(ivar) );
    error(errmsg)
  end
  //
  // Checking size of input arguments
  if ( or(size(funname) <> [1 1]) ) then
    strcomp = strcat(string(size(funname))," ")
    errmsg = msprintf(gettext("%s: Expected a scalar for input argument %s at input #%d, but got [%s] instead."), "specfun_checkflint", "funname" , 1 , strcomp );
    error(errmsg)
  end
  if ( or(size(varname) <> [1 1]) ) then
    strcomp = strcat(string(size(varname))," ")
    errmsg = msprintf(gettext("%s: Expected a scalar for input argument %s at input #%d, but got [%s] instead."), "specfun_checkflint", "varname" , 3 , varname );
    error(errmsg)
  end
  if ( or(size(ivar) <> [1 1]) ) then
    strcomp = strcat(string(size(ivar))," ")
    errmsg = msprintf(gettext("%s: Expected a scalar for input argument %s at input #%d, but got [%s] instead."), "specfun_checkflint", "ivar" , 4 , strcomp );
    error(errmsg)
  end
  //
  errmsg = []
  if ( or(floor(var)<>var) ) then
    k = find ( floor(var)<>var , 1 )
    errmsg = msprintf(gettext("%s: Wrong content of input argument %s at input #%d: expected a floating point integer, but entry #%d is equal to %s."),funname,varname,ivar,k,string(var(k)))
    error ( errmsg)
  end
endfunction

