// Copyright (C) 2010 - DIGITEO - Michael Baudin
//
// This file must be used under the terms of the CeCILL.
// This source file is licensed as described in the file COPYING, which
// you should have received as part of this distribution.  The terms
// are also available at
// http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt

//
// assert_close --
//   Returns 1 if the two real matrices computed and expected are close,
//   i.e. if the relative distance between computed and expected is lesser than epsilon.
// Arguments
//   computed, expected : the two matrices to compare
//   epsilon : a small number
//
function flag = assert_close ( computed, expected, epsilon )
  if expected==0.0 then
    shift = norm(computed-expected);
  else
    shift = norm(computed-expected)/norm(expected);
  end
  if shift < epsilon then
    flag = 1;
  else
    flag = 0;
  end
  if flag <> 1 then pause,end
endfunction
//
// assert_equal --
//   Returns 1 if the two real matrices computed and expected are equal.
// Arguments
//   computed, expected : the two matrices to compare
//   epsilon : a small number
//
function flag = assert_equal ( computed , expected )
  if computed==expected then
    flag = 1;
  else
    flag = 0;
  end
  if flag <> 1 then pause,end
endfunction

function d = assert_computedigits ( computed , expected )
  // Returns the number of significant digits in computed result.
  //
  // Calling Sequence
  //   d = assert_computedigits ( computed , expected )
  //
  // Parameters
  //   computed : a matrix of doubles, the computed value
  //   expected : a matrix of doubles, the expected value
  //   d : a matrix of doubles, the number of significant digits.
  //
  // Description
  //   Computes the number of significant digits in a computed 
  //   result with respect to a nonzero expected result, using the formula:
  //
  //<latex>
  //\begin{eqnarray}
  //d = -\log_{10} ( r )
  //\end{eqnarray}
  //</latex>
  //
  // where r is the relative error defined by 
  //
  //<latex>
  //\begin{eqnarray}
  //r = \frac{|computed - expected|}{|expected|}
  //\end{eqnarray}
  //</latex>
  // 
  // The number of significant digits is between dmin = 0 and dmax = -log10(2^(-53)) which 
  // is approximately 15.95.
  //
  // If expected is equal to computed, then d is set to its maximum value, i.e. dmax.
  // If not, if expected is zero and computed is nonzero, then d is set to its minimum 
  // value, i.e. dmin=0.
  //
  // The values of computed and expected cannot be exchanged, since the relative 
  // error is based on the expected value.
  //
  // The computation separates the real part and the imaginary parts of the 
  // values.
  // The returned number of digits is the minimum of the number of digits for the 
  // real and imaginary parts.
  //
  // TODO : use computedigits inside a assert_digits ( computed , expected , dmin ) function
  //
  // TODO : add a comptype option to make the comparison "matrix" (current is "element")
  //
  // Examples
  //   d = assert_computedigits ( 1 , 1 ) // d = 15.95459
  //   d = assert_computedigits ( 0 , 0 ) // d = 15.95459
  //   d = assert_computedigits ( 1 , 0 ) // d = 0
  //   d = assert_computedigits ( 0 , 1 ) // d = 0
  //   d = assert_computedigits ( 3.1415926 , %pi ) // d = 7.7680678
  //   d = assert_computedigits ( [0 0 1 1] , [0 1 0 1] ) // d = [15.95459 0 0 15.95459]
  //   d = assert_computedigits(ones(3,2),ones(3,2)) // d = 15.95459 * ones(3,2)
  //   d = assert_computedigits([%nan %nan %nan %nan],[%nan %inf -%inf 0]) // d = [15.95459 0 0 0]
  //   d = assert_computedigits([%inf %inf %inf %inf],[%nan %inf -%inf 0]) // d = [0 15.95459 0 0]
  //   d = assert_computedigits([-%inf -%inf -%inf -%inf],[%nan %inf -%inf 0]) // d = [0 0 15.95459 0]
  //   d = assert_computedigits([0 0 0 0],[%nan %inf -%inf 0]) // d = [0 0 0 15.95459]
  //

  [lhs,rhs]=argn()
  if ( rhs <> 2 ) then
    errmsg = sprintf ( gettext ( "%s: Unexpected number of input arguments : %d provided while %d are expected.") , "assert_computedigits" , rhs , 2 )
    error(errmsg)
  end
  //
  // Check types of variables
  if ( typeof(computed) <> "constant" ) then
    errmsg = sprintf ( gettext ( "%s: Unexpected type of input argument #%d : variable %s has type %s while %s is expected.") , "assert_computedigits" , 1 , "computed" , typeof(computed) , "constant" )
    error(errmsg)
  end
  if ( typeof(expected) <> "constant" ) then
    errmsg = sprintf ( gettext ( "%s: Unexpected type of input argument #%d : variable %s has type %s while %s is expected.") , "assert_computedigits" , 2 , "expected" , typeof(expected) , "constant" )
    error(errmsg)
  end
  //
  // Check sizes of variables
  if ( size(expected) <> size(computed) ) then
    errmsg = sprintf ( gettext ( "%s: The size of computed is [%d,%d] while the size of expected is [%d,%d].") , "assert_computedigits" , size(expected,"r") , size(expected,"c") , size(computed,"r") , size(computed,"c") )
    error(errmsg)
  end
  //
  nre = size(expected,"r")
  nce = size(expected,"c")
  // Update shape
  expected = expected (:)
  computed = computed (:)
  //
  dmin = 0
  dmax = -log10(2^(-53))
  //
  d = zeros(expected)
  //
  n = size(expected,"*")
  for i = 1 : n
    dre = computedigits_data ( real(expected(i)) , real(computed(i)) )
    dim = computedigits_data ( imag(expected(i)) , imag(computed(i)) )
    d(i) = min(dre,dim)
  end
  //
  // Reshape
  d = matrix(d,nre,nce)
endfunction
function d = computedigits_data ( expected , computed )
    if ( isnan(expected) & isnan(computed) ) then
      d = dmax
    elseif ( isnan(expected) & ~isnan(computed) ) then
      d = dmin
    elseif ( ~isnan(expected) & isnan(computed) ) then
      d = dmin
      // From now, both expected and computed are non-nan
    elseif ( expected == 0 & computed == 0 ) then
      d = dmax
    elseif ( expected == 0 & computed <> 0 ) then
      d = dmin
      // From now, expected is non-zero
    elseif ( expected == computed ) then
      d = dmax
      // From now, expected and computed are different
    elseif ( expected == %inf & computed <> %inf ) then
      d = dmin
    elseif ( expected == -%inf & computed <> -%inf ) then
      d = dmin
      // From now, neither of computed, nor expected is infinity
    else
      d = max ( -log10 ( abs(computed-expected) / abs(expected) ) , dmin )
    end
endfunction


function rtol = assert_condition2reltol ( condition )
  // Suggests a relative error, computed from the condition number.

  dmax = -log10(2^(-53))
  expectedlostdigits = min(max(log10(condition),0),dmax)
  requireddigits = dmax - expectedlostdigits
  rtol = 10^-requireddigits
endfunction
function reqdigits = assert_cond2reqdigits ( condition , offset )
  // Returns the number of required digits from the condition number and an offset
  // offset : if offset positive, it represents the number of accepted lost digits.
  dmax = -log10(2^(-53))
  lostdigits = min(max(log10(condition),0),dmax) + offset
  reqdigits = dmax - lostdigits
endfunction

function mat = dataset_read ( filename , commenttag , sep , verbose )
    v = mgetl(filename)
    nrows = size(v,"r")
    if ( verbose ) then
        mprintf("Dataset : %s\n",filename)
        mprintf("Number of lines : %d\n",nrows)
    end
    //
    // Remove comment lines
    for i = 1 : size(v,'*')
        line = v(i);
        firstchar = part(line,1)
        if ( firstchar == commenttag ) then
            v(i) = ""
        end
    end
    //
    // Remove empty lines
    v(v == '') = [];
    //
    ns = length(sep);
    mat = [];
    ki = 1;
    for i = 1 : size(v,'*')
        line = v(i);
        K = [strindex(line, sep)];
        ki = 1;
        row = [];
        for k = K
            row = [row, part(line,ki:k-1)];
            ki = k + ns;
        end
        row = [row, part(line, ki:length(line))];
        if (i > 1) then
            if ( size(row,2) > size(mat,2) ) then
                mat($,size(row,2)) = '';
            elseif ( size(row,2) < size(mat,2) ) then
                row(1, size(mat,2)) = '';
            end
        end
        mat = [mat; row];
    end
    mat = strsubst(mat,"I","%i")
endfunction


////////////////////////////////////////////////////////////////////////
// 
// Check robustness
computed = specfun_expm1([]);
assert_equal ( computed , [] );
//
// IEEE values
computed = specfun_expm1(0);
assert_equal ( computed , 0 );
//
computed = specfun_expm1(%inf);
assert_equal ( computed , %inf );
// 
computed = specfun_expm1(%nan);
assert_equal ( isnan(computed) , %t );
// 
computed = specfun_expm1(-%inf);
assert_equal ( computed , -1 );
//
// A complex
computed = specfun_expm1(1+%i);
expected = 0.468693939915885 + 2.287355287178842 * %i;
assert_close ( real(computed) , real(expected) , 2*%eps );
assert_close ( imag(computed) , imag(expected) , 2*%eps );
//
// A row vector
computed = specfun_expm1(1:4);
expected = [
  1.718281828459045235
  6.389056098930650227
  19.08553692318766774
  53.59815003314423907
]';
assert_close ( computed , expected , %eps );
//
// A column vector
computed = specfun_expm1((1:4)');
expected = [
  1.718281828459045235 
  6.389056098930650227
  19.08553692318766774
  53.59815003314423907
];
assert_close ( computed , expected , %eps );
//
// A matrix
computed = specfun_expm1(ones(2,3));
expected = 1.718281828459045235 * ones(2,3);
assert_close ( computed , expected , %eps );
////////////////////////////////////////////////////////////
//
// Some simple accuracy tests
// [x fexact]
table = [
1.e-3  1.00050016670834166e-3
1.e-6  1.00000050000016666e-6
1.e-9  1.00000000050000000e-9
1.e-12 1.00000000000050000e-12
1.e-20 1.00000000000000000e-20
-1.e-3  -9.9950016662500833194e-4
-1.e-6  -9.9999950000016666662e-7
-1.e-9  -9.9999999950000000016e-10
-1.e-12 -9.9999999999950000000e-13
-1.e-20 -9.9999999999999999999e-21
];
ntests = size(table,"r");
x = table(:,1);
computed = specfun_expm1(x);
expected = table(:,2);
digits = assert_computedigits ( computed , expected );
requireddigits = 15.5;
for i = 1 : ntests
  if ( digits(i) < requireddigits ) then
    ok = "!!!FAIL!!!";
  else
    ok = "PASS";
  end
  if ( %f ) then
    mprintf("Test #%2d/%2d, x=%s, y=%s, e=%s, d=%3.1f, %s\n", ..
      i, ntests, sci2exp(x(i)), sci2exp(computed(i)), sci2exp(expected(i)) , digits(i), ok);
  end
  assert_equal ( ok , "PASS" );
end




